use {
	gtk4 :: {
		FlowBox, HeaderBar, Image, Justification, Label, SelectionMode,
		Align :: * ,
		Orientation :: * ,
		gdk :: Texture ,
		prelude :: * ,
	} ,
	libadwaita :: { * , ViewSwitcherPolicy :: * } ,
	std :: path :: * ,
} ;

use crate :: collection :: * ;



pub fn create_window (
	application : & Application ,
	header_bar : & HeaderBar ,
	collection_view_stack : & ViewStack ,
) -> ApplicationWindow {
	let content = gtk4 :: Box :: builder () . orientation (Vertical) . build () ;
	content . append (header_bar) ;
	content . append (collection_view_stack) ;

	ApplicationWindow :: builder ()
		. application (application)
		. content ( & content )
		. title ("Zoödex")
		. build ()
}

pub fn create_header_bar ( collection_view_stack : & ViewStack ) -> HeaderBar {
	HeaderBar :: builder ()
		. title_widget (
			& create_collection_view_switcher ( collection_view_stack ) ,
		)
		. build ()
}

pub fn create_collection_view_switcher ( stack : & ViewStack ) -> ViewSwitcher {
	ViewSwitcher :: builder ()
		. policy (Wide)
		. stack (stack)
		. build ()
}

pub struct FilmsFlowBox { widget : FlowBox }
pub struct SeriesFlowBox { widget : FlowBox }

impl FilmsFlowBox {
	pub fn new ( films : & [Film] ) -> Self {
		let widget = create_collection_flow_box () ;

		for film in films {
			widget . append ( & create_film_item (film) ) ;
		}

		Self { widget }
	}

	pub fn set_films ( & self , films : & [Film] ) {
		self . widget . remove_all () ;
		for film in films {
			self . widget . append ( & create_film_item (film) ) ;
		}
	}

	pub fn get_widget ( & self ) -> & FlowBox { & self . widget }
}
impl SeriesFlowBox {
	pub fn new ( series : & [Series] ) -> Self {
		let widget = create_collection_flow_box () ;

		for series in series {
			widget . append ( & create_series_item (series) ) ;
		}

		Self { widget }
	}

	pub fn set_series ( & self , series : & [Series] ) {
		self . widget . remove_all () ;
		for series in series {
			self . widget . append ( & create_series_item (series) ) ;
		}
	}

	pub fn get_widget ( & self ) -> & FlowBox { & self . widget }
}

fn create_collection_flow_box () -> FlowBox {
	FlowBox :: builder ()
		. orientation (Horizontal)
		. homogeneous (true)
		. selection_mode ( SelectionMode :: None )
		. build ()
}

pub fn create_film_item ( film : & Film ) -> gtk4 :: Box {
	create_collection_item (
		film . name . as_str () ,
		film . original_name . as_deref () ,
		film . poster_file_path . as_deref () ,
		& create_film_details (film) ,
	)
}
pub fn create_series_item ( series : & Series ) -> gtk4 :: Box {
	create_collection_item (
		series . name . as_str () ,
		series . original_name . as_deref () ,
		series . poster_file_path . as_deref () ,
		& create_series_details (series) ,
	)
}

fn create_film_details ( film : & Film ) -> gtk4 :: Box {
	let container = gtk4 :: Box :: builder ()
		. orientation (Horizontal)
		. halign (Center)
		. spacing (20)
		. build () ;

	container . append (
		& Label :: builder () . label ( & film . release_date ) . build ()
	) ;
	container . append (
		& Label :: builder () . label ( format ! ( "{}m" , film . runtime_minutes ) ) . build ()
	) ;

	container
}
fn create_series_details ( series : & Series ) -> gtk4 :: Box {
	let container = gtk4 :: Box :: builder ()
		. orientation (Horizontal)
		. halign (Center)
		. spacing (20)
		. build () ;

	// TODO

	container
}

fn create_collection_item (
	name : & str ,
	original_name : Option < & str > ,
	poster_file_path : Option < & Path > ,
	details_widget : & gtk4 :: Box ,
) -> gtk4 :: Box {
	let container = gtk4 :: Box :: builder ()
		. orientation (Vertical)
		. margin_top (20)
		. margin_bottom (20)
		. build () ;

	if let Some (poster_file_path) = poster_file_path {
		if let Ok (poster_texture) = Texture :: from_filename (poster_file_path) {
			container . append (
				& Image :: builder ()
					. paintable ( & poster_texture )
					. width_request (300)
					. height_request (300)
					. margin_bottom (10)
					. build ()
			) ;
		}
	}

	container . append (
		& Label :: builder ()
			. label ( format ! ( "<span size='large' weight='bold'>{}</span>" , name ) )
			. use_markup (true)
			. justify ( Justification :: Center )
			. wrap (true)
			. max_width_chars (1)  // Not the actual limit, used instead to wrap more aggressively
			. build ()
	) ;

	if let Some (original_name) = original_name {
		container . append (
			& Label :: builder ()
				. label (original_name)
				. justify ( Justification :: Center )
				. wrap (true)
				. max_width_chars (1)  // Not the actual limit, used instead to wrap more aggressively
				. build ()
		) ;
	}

	container . append (details_widget) ;

	container
}
